#!/usr/bin/env python
import os
import select
import sys
import string
# Copyright (C) 2001 Aaron Lav, asl2@pobox.com

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


# The following discussion is almost all about Linux.  Sorry for
# ignoring portability issues, but I know of no good portable
# way to do what I want.  Patches for your system will be accepted.

# This module suffers from granularity problems on my machine (Linux x86
# 2.2.12).  Both select () and nanosleep () are implemented in terms
# of schedule_timeout, which deals in jiffies.  Jiffies are at 100 Hz,
# so we can either go at 100 cps, fractions thereof, or full-speed (if
# the delay is smaller than can be expressed in struct timeval, and
# thus the delay rounds down to 0.  Gallmeister's POSIX.4 book claims
# in the nanosleep man page that 100 Hz is a fairly common granularity.

# Alternatives: if we run with real-time priority (see kernel/sched.c)
# and use nanosleep, then the kernel will busy-wait for us.  Alternately,
# we could use /dev/rtc (see drivers/char/rtc.c), but random users aren't
# allowed to set periodic interrupts faster than 64 Hz.  (The max is
# 8192 Hz, so we can't simulate serial ports at 115000 feeding a 56k
# modem with compression turned on: but we don't simulate V.42bis compression
# anyway.)

# Stupid trick: if we have a spare serial port, and can set loopback
# mode (on an 8250 or 16[45]50, bit 4 of the MCR: on an 8530,
# it looks like WR14, bit 4), we could stuff all our data into the
# write side, and get it clocked out at exactly the right rate.
# We could also just tx data and wait for the write buffer empty
# interrupt, but some UARTs have a hidden shift register (out
# of which the bits are actually shifted), and there's no way
# to tell how many bits it has in it.  

# Even less desirable: we could busywait while calling gettimeofday
# (on x86 at least, this claims to have microsecond or better than
# microsecond resolution and precision "on fast x86 machines with TSC."
# (linux/arch/i386/kernel/time.c), or we could write calibratedly slow
# X or console output drivers.

# We could take a degree of line impairment, and simulate V.32 and better
# equalizer retraining, V.42 NAK and retransmit delays, but it would be
# wrong: the whole point of bvwb is allowing web designers to exploit
# predictable delays.  (If there were an HTML extension for degree of
# impairment, that would be different, since designers could choose
# to incorporate an aleatoric element.)

class slow_output:
    def __init__ (self):
        self._file = open ("/dev/tty", "w", 0)
        # must open a new file instead of reusing stdout because Python
        # doesn't allow changing the buffering mode of a currently open file,
        # and stdout is linebuffered by default.
        # The claim is that on some systems, this may dump core.

    def set_delay_secs (self,delay):
        self._delay_secs = delay

    def write(self, text):
        for c in text:
            self._file.write (c)
            if self._delay_secs > 1e-5:
                select.select ([],[],[],self._delay_secs)

if __name__ == '__main__':
    out = slow_output ()
    out.set_delay_secs (1/30.0)
    if len(sys.argv) >= 2:
        arg = string.atof (sys.argv[1])
        if arg <> 0:
            out.set_delay_secs (1/arg)
    if len(sys.argv) == 3:
        file = open(sys.argv[2])
        text = file.read ()
    else:
        text = "Cuidado, cuidado, cuidado, llamas!\n"
        assert (len(text) < 50)
        text = string.ljust (text, 50) * 2
        assert (len(text) == 100)
        # round numbers like 100 make it easy even for me to do calcs
        # in my head
    out.write (text)
    print 1/out._delay_secs, len(text), len(text)*out._delay_secs
    # calculate expected time for nominal baud rate, easy to time
    # from command line and compare
